package ru.dbotthepony.kstarbound.defs.item

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.defs.ThingDescription
import ru.dbotthepony.kstarbound.defs.util.enrollMap
import ru.dbotthepony.kstarbound.io.json.builder.BuilderAdapter
import ru.dbotthepony.kstarbound.io.json.builder.INativeJsonHolder
import ru.dbotthepony.kstarbound.io.json.builder.JsonBuilder
import ru.dbotthepony.kstarbound.io.json.builder.JsonPropertyConfig
import ru.dbotthepony.kstarbound.io.json.builder.JsonIgnoreProperty
import ru.dbotthepony.kstarbound.util.NotNullVar

@JsonBuilder
open class ItemPrototype : IItemDefinition, INativeJsonHolder {
	@JsonIgnoreProperty
	final override var shortdescription: String = "..."
	@JsonIgnoreProperty
	final override var description: String = "..."

	final override var itemName: String by NotNullVar()
	final override var price: Long = 0L
	final override var rarity: ItemRarity = ItemRarity.COMMON
	final override var category: String? = null
	final override var inventoryIcon: ImmutableList<IItemDefinition.InventoryIcon>? = null
	final override var itemTags: ImmutableList<String> = ImmutableList.of()
	final override var learnBlueprintsOnPickup: ImmutableList<String> = ImmutableList.of()
	final override var maxStack: Long = 9999L
	final override var eventCategory: String? = null
	final override var consumeOnPickup: Boolean = false
	final override var pickupQuestTemplates: ImmutableList<String> = ImmutableList.of()
	final override var tooltipKind: String = "normal"
	final override var twoHanded: Boolean = false
	final override var radioMessagesOnPickup: ImmutableList<String> = ImmutableList.of()
	final override var fuelAmount: Long? = null

	@JsonPropertyConfig(isFlat = true)
	var descriptionData: ThingDescription by NotNullVar()

	@JsonIgnoreProperty
	var json: Map<String, Any> = mapOf()

	final override fun acceptJson(json: MutableMap<String, Any>) {
		this.json = json
	}

	open fun assemble(): IItemDefinition {
		return ItemDefinition(
			descriptionData = descriptionData,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			json = enrollMap(json),
		)
	}
}
