package ru.dbotthepony.kstarbound.io.json.factory

import com.google.common.collect.ImmutableList
import com.google.common.collect.ImmutableMap
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter

class ImmutableMapTypeAdapter<V>(val elementAdapter: TypeAdapter<V>) : TypeAdapter<ImmutableMap<String, V>>() {
	override fun write(out: JsonWriter, value: ImmutableMap<String, V>?) {
		if (value == null) {
			out.nullValue()
			return
		}

		out.beginObject()

		for ((k, v) in value) {
			out.name(k)
			elementAdapter.write(out, v)
		}

		out.endObject()
	}

	override fun read(reader: JsonReader): ImmutableMap<String, V>? {
		if (reader.peek() == JsonToken.NULL)
			return null

		reader.beginObject()

		val builder = ImmutableMap.Builder<String, V>()

		while (reader.peek() != JsonToken.END_OBJECT) {
			builder.put(reader.nextName(), elementAdapter.read(reader) ?: throw JsonSyntaxException("Nulls are not allowed, near ${reader.path}"))
		}

		reader.endObject()
		return builder.build()
	}
}
